"""
- A @dataclass describes the application's configuration
- ConfigStore manages the Structured Config
- cfg is duck typed as a MySQLConfig instead of a DictConfig
"""
from dataclasses import dataclass
from typing import Any, List

import hydra
import torch
from hydra.core.config_store import ConfigStore


@dataclass
class BenchmarkConfig:
    # models info
    dir_generators: str

    # dataset info
    img_resolution: int
    num_classes: int
    num_channels: int
    classifier_name: str
    ood_datadir: str

    # expert info
    cl_model: torch.nn.Module
    exert_model: torch.nn.Module

    # cl benchmark info
    name: str
    num_experiences: int
    benchmark_instance: Any


@dataclass
class ModeConfig:
    debug_mode: bool
    run_id: int
    log_interactive: bool
    experiment_type: str


@dataclass
class ServerConfig:
    logdir: str
    datadir: str
    num_gpus: int


@dataclass
class OODDatasetConfig:
    name: str
    ood_data: torch.utils.data.Dataset


@dataclass
class ModelSelectionConfig:
    num_trials: int
    search_space: List[Any]


@dataclass
class StrategyConfig:
    temperature: float
    train_epochs: int
    latent_proj_lambda: float
    latent_proj_init: str
    plugins: List[Any]
    init_from_first: bool
    latent_layers_infos: List[Any]
    model_selection: ModelSelectionConfig


@dataclass
class ExperimentConfig(ModeConfig, StrategyConfig):
    # defaults
    server: ServerConfig
    benchmark: BenchmarkConfig
    mode: ModeConfig
    ood_dataset: OODDatasetConfig

    # experiment specific
    experiment_name: str
    optimizer: Any
    strategy: StrategyConfig


cs = ConfigStore.instance()
# Registering the Config class with the name 'config'.
# cs.store(name="config", node=ExperimentConfig)
