import torch
import torch.nn.functional as F


def masked_kd_loss(student_logits, teacher_logits, active_units=None, temperature=1.0):
    """
    Compute distillation loss between output of the current model and
    and output of the previous (saved) model.
    """
    if active_units is not None:
        student_logits = student_logits[:, active_units]
        teacher_logits = teacher_logits[:, active_units]

    curr_out = student_logits / temperature
    prev_out = teacher_logits / temperature

    res = F.kl_div(
        F.log_softmax(curr_out, dim=1),
        F.log_softmax(prev_out, dim=1),
        log_target=True,
        reduction="batchmean")
    return res


def masked_ce_loss(y_pred, y_true, active_units=None, mask_value=-10):
    mask = torch.ones(y_pred.shape[-1])
    if active_units:
        mask[active_units] = 0
    else:
        mask[:] = 0

    mask = mask.bool()
    # print(mask.tolist())
    y_pred[:, mask] = mask_value
    return F.cross_entropy(y_pred, y_true)
