import logging

import random

import datetime
import sys
import os
import ray

from dac.utils import maybe_set_gpus
from dac.hydra_conf import ExperimentConfig

os.environ["PYTHONPATH"] = os.environ["PYTHONPATH"] + ':$ANON_HOME_DIR/avalanche'
os.environ["PYTHONPATH"] = os.environ["PYTHONPATH"] + ':$ANON_HOME_DIR/dac_project'
sys.path.append("$ANON_HOME_DIR/dac_project")
sys.path.append("$ANON_HOME_DIR/avalanche")

import hydra

import copy
import os

from hydra.utils import instantiate
from omegaconf import DictConfig, OmegaConf
from ray import tune
from ray.tune.schedulers import ASHAScheduler
from dac.training.configs.train_kd_latent import train_config as train_config_kd_latent
from dac.training.configs.train_kd_curr_data import train_config as train_config_kd_curr_data
from dac.training.configs.train_macl import train_config as train_config_macl
from dac.training.configs.train_macl_curr_data import train_config as train_config_macl_curr_data

global train_config_foo

pil_logger = logging.getLogger('PIL')
pil_logger.setLevel(logging.INFO)


def train_foo_debug(cfg):
    for i in range(10):
        tune.report(train_stream_acc=random.random())


def override_hps_and_train(hps):
    local_config = override_config(hps)
    train_config_foo(local_config)


def override_config(hps):
    local_config = copy.deepcopy(global_config)
    for k, v in hps.items():
        ks = k.split('.')
        d = local_config
        for k in ks[:-1]:
            d = d[k]
        d[ks[-1]] = v
    return local_config


@hydra.main(config_path='CONFIGS', config_name='base.yaml')
def main(config: ExperimentConfig):
    if config.debug_mode:
        pass

    maybe_set_gpus(config.server.num_gpus)
    global train_config_foo

    if config.train_config == "cl_mt_latproj":
        train_config_foo = train_config_kd_latent
    elif config.train_config == "seq_kd_curr":
        train_config_foo = train_config_kd_curr_data
    elif config.train_config == "indip_mt_latproj":
        train_config_foo = train_config_macl
    elif config.train_config == "indip_kd_curr":
        train_config_foo = train_config_macl_curr_data
    elif config.train_config == "debug":
        train_config_foo = train_foo_debug
    else:
        raise ValueError("Unkown training config.")

    if config.experiment_type == "single_run":
        print("------------------ SINGLE-RUN.")
        train_config_foo(config)
    elif config.experiment_type == "model_assessment":
        print("------------------ MODEL-ASSESSMENT.")
        model_assessment(config)
    elif config.experiment_type == "model_selection":
        print("------------------ MODEL-SELECTION.")
        analysis = model_selection(config)
        best_hps = analysis.get_best_config(metric="train_stream_acc", mode="max")
        model_assessment(override_config(best_hps))
    else:
        raise ValueError("Unknown config.experiment_type: ", config.experiment_type)


def model_assessment(config):
    global train_config_foo

    sdate = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    exp_name = 'assess_' + sdate
    os.makedirs(exp_name, exist_ok=config.debug_mode)
    os.chdir(exp_name)

    for runid in range(5):
        local_config = copy.deepcopy(config)
        local_config.run_id = runid
        train_config_foo(local_config)


if __name__ == "__main__":
    main()
